unit hklister;
{ THotkeyLister beta version 1 (11/4/96)
 (c) 1996, by Maggie Owens
 mmowens@panix.com
 http://www.panix.com/~mmowens

 This componennt is provided free, without warranty or guarantee of any
 kind -- use it at your own risk.  The author takes no responsibility for
 any problems it may cause.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Menus, Dsgnintf;

type
  TmoHotkeyLister = class(TTreeView)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FMenuForm: TForm;
    FDoFullExpand: Boolean;
    stl: TStringList;
    function DelAnd(st1: String): String;
    procedure AddMenuItem(const ttn: TTreeNode; const mi1: TMenuItem);
  public
    { Public declarations }
    property MenuForm: TForm read FMenuForm write FMenuForm;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    { Published declarations }
    property DoFullExpand: Boolean read FDoFullExpand write FDoFullExpand default True;
  end;

  THKListerComponentEditor = class(TComponentEditor)
  private
  protected
  public
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
    procedure Edit; override;
  published
  end;

procedure Register;

implementation

{ TmoHotkeyLister }

constructor TmoHotkeyLister.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  stl := TStringList.Create;
  stl.Sorted := True;
  FDoFullExpand := True;
end;

destructor TmoHotkeyLister.Destroy;
begin
  stl.Free;
  inherited destroy;
end;

procedure TmoHotkeyLister.Execute;
var
  i: Integer;
begin
  if FMenuForm = nil then
    raise Exception.Create('No MenuForm defined');
  Items.Clear;
  with fMenuForm do
    for i := 0 to ComponentCount -1 do
      if (Components[i] is TMenuItem) and
        (TMenuItem(Components[i]).Count > 1) then begin
        AddMenuItem(nil, TMenuItem(Components[i]))
    end;
  if FDoFullExpand then
    FullExpand;
end;

procedure TmoHotkeyLister.AddMenuItem(const ttn: TTreeNode; const mi1: TMenuItem);
var
  j, k, n: Integer;
  mi: TMenuItem;
  tn: TTreeNode;
  st: String;
begin
  with FMenuForm do begin
    if ttn = nil then
      Items.Add(ttn, DelAnd(mi1.Caption) + ' (' + mi1.Hint + '):')
    else
      Items.AddChild(ttn, DelAnd(mi1.Caption) + ' (' + mi1.Hint + '):');
    tn := Items[Items.Count-1];
    k := 0; // # of sub-menu items of this item to be displayed
    for j := 0 to mi1.Count -1 do
      if stl.Find(mi1.Items[j].Name, n) then
        continue
      else if mi1.Items[j].Count > 1 then
        AddMenuItem(tn, mi1.Items[j])
      else if (mi1.Items[j].ShortCut <> 0) then begin
        Inc(k);
        mi := mi1.Items[j];
        if mi.Hint <> '' then
          st := ' (' + mi.Hint + '):'
        else
          st := ':';
        Items.AddChild(tn, DelAnd(mi.Caption) + st + ShortCutToText(mi.ShortCut));
        stl.Add(mi.Name);
      end;
    if k = 0 then // no sub-menu items were added: delete the parent.
      Items.Delete(tn)
  end
end;

function TmoHotkeyLister.DelAnd(st1: String): String;
// Gets the '&' out of a string
var
  i: Integer;
begin
  Result := '';
   for i := 1 to Length(st1) do
    if st1[i] <> '&' then
      Result := Result + st1[i];
end;

{ THKListerComponentEditor }
function THKListerComponentEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function THKListerComponentEditor.GetVerb(Index: Integer): string;
begin
  case index of
    0: Result := '(c) 1996, Maggie Owens';
  end;
end;

procedure THKListerComponentEditor.ExecuteVerb(Index: Integer);
begin
  case index of
    0: MessageDlg('Version 1.0' + #13 + 'http://www.panix.com/~mmowens',
       mtInformation, [mbOk], 0);
  end;
end;

procedure THKListerComponentEditor.Edit;
begin
  (Component as TmoHotkeyLister).Execute;
end;


procedure Register;
begin
  RegisterComponents('Samples', [TmoHotkeyLister]);
  RegisterComponentEditor(TmoHotkeyLister, THKListerComponentEditor)
end;

end.
